package com.appdatatracker.networking

import android.content.Context
import com.appdatatracker.utils.Constants
import com.appdatatracker.utils.Constants.provideAppContext
import okhttp3.Interceptor
import okhttp3.OkHttpClient
import okhttp3.Response
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory


fun provideLoggingInterceptor(): HttpLoggingInterceptor { //for logs
    val logging = HttpLoggingInterceptor()
    logging.level = HttpLoggingInterceptor.Level.BODY
    return logging
}

var appInterceptor: Interceptor? = null
fun provideHttpClient(): OkHttpClient {
    val httpClient = OkHttpClient.Builder()
    if (appInterceptor == null)
        appInterceptor = MyAppInterceptor(provideAppContext())

    httpClient.addInterceptor(appInterceptor!!)
    httpClient.addInterceptor(provideLoggingInterceptor())
    return httpClient.build()
}


fun provideApiProvider(): Retrofit {
    return Retrofit.Builder()
        .baseUrl(Constants.BASE_URL)
        .client(provideHttpClient())
        .addConverterFactory(GsonConverterFactory.create())
        .build()
}


fun provideApiService(retrofit: Retrofit): ApiService {
    return retrofit.create(ApiService::class.java)
}


class MyAppInterceptor(val context: Context) : Interceptor {

    override fun intercept(chain: Interceptor.Chain): Response {

        var request = chain.request()
        val headers = request.headers.newBuilder()
            .add("Content-Type", "application/json")
            .add("Accept", "application/json")
            .build()

        request = request.newBuilder().headers(headers).build()

        return chain.proceed(request)
    }
}